/*  
   Nokia Display N5110 Interfacing with ATmega 16
   http://www.electronicwings.com
*/

//#define F_CPU 8000000UL
#include <stdio.h>
#include <avr/io.h>
#include <util/delay.h>
#include <string.h>
#include "spi.h"
#include "Font.h"

#define MOSI 5
#define MISO 6
#define SCK 7
#define SS 4
#define DC 1
#define RST 0


void adc_init(void)
{
	ADMUX = 0;
	ADMUX |= (1 << MUX0);          
	ADMUX |= (1 << REFS0);					// AVCC with external capacitor at AREF pin      	
	ADCSRA = 0;
	ADCSRA |= (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0);   //set clock prescaler at 128 
	ADCSRA |= (1 << ADEN);					// enable ADC
	
}

uint16_t adc_read(uint8_t channel)
{
	
	ADMUX &= 0xF0;                    	//Clear the older channel that was read
	ADMUX |= channel;                	//Defines the new ADC channel to be read
	ADCSRA |= (1<<ADSC);               	//Starts a new conversion
	while(ADCSRA & (1<<ADSC));         	//Wait until the conversion is done
	return ADC;                   		//Returns the ADC value of the chosen channel
}

uint16_t calibrate(void)
{
	uint16_t sum = 0;
	uint16_t average = 0;
	for(int i = 0; i < 200; i++) {
		sum += adc_read(0);
		_delay_ms(150);
	}
	average = (uint16_t)(sum/200);
	return average;
}

uint16_t convert(uint16_t mq3Value, uint16_t mq3Normal, uint16_t alcNormal)
{
	uint16_t alcValue;
	// conversia din voltaj in mg/L in aer respirat.
	alcValue = (uint16_t)((mq3Value * alcNormal)/mq3Normal);

	return alcValue;
}

void SPI_SS_Enable()												/* Enable the SS pin to logic zero */		
{
	PORTB &= ~(1<<SS);
}

void SPI_SS_Disable()												/* disable the SS pin to logic one */
{
	PORTB |= (1<<SS);
}

void SPI_Init()
{
	DDRB = (1<<MOSI)|(1<<SCK)|(1<<SS)|(1<<RST)|(1<<DC);				/* Set MOSI and SCK output, all others input */
	PORTB |= (1<<RST);												/* define reset pin as a output */
	DDRB &= ~(1<<MISO);												/* define MISO pin as a input */
	PORTB |= (1<<SS);												/* Make SS pin as a output */
	SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR0);							/* Enable SPE, MSTR and SPR0 bit in SPI control register */
}

char SPI_Read()
{
	SPDR = 0xFF;
	while(!(SPSR & (1<<SPIF)));										/* wait here while SPIF flag set */
	return(SPDR);													/* Return the SPDR value */
}

void SPI_Write(char write_data)										
{
	SPDR = write_data;													
	while(!(SPSR & (1<<SPIF)));										/* wait here while SPIF flag set */
}

void N5110_Cmnd(char DATA) 
{
	PORTB &= ~(1<<DC);  /* make DC pin to logic zero for command operation */
	SPI_SS_Enable();  /* enable SS pin to slave selection */	
	SPI_Write(DATA);  /* send data on data register */
	PORTB |= (1<<DC);  /* make DC pin to logic high for data operation */
	SPI_SS_Disable();
}

void N5110_Data(char *DATA)
{
	PORTB |= (1<<DC);  /* make DC pin to logic high for data operation */
	SPI_SS_Enable();  /* enable SS pin to slave selection */
	int lenan = strlen(DATA);  /* measure the length of data */
	for (int g=0; g<lenan; g++)
	{
		for (int index=0; index<5; index++)
		{
			SPI_Write(ASCII[DATA[g] - 0x20][index]);  /* send the data on data register */			
		}
		SPI_Write(0x00);
	}							
	SPI_SS_Disable();									
}

void N5110_Reset()  /* reset the Display at the beginning of initialization */
{
	PORTB &= ~(1<<RST);
	_delay_ms(100);
	PORTB |= (1<<RST);
}

void N5110_init()
{
	N5110_Reset();  /* reset the display */
	N5110_Cmnd(0x21);  /* command set in addition mode */
	N5110_Cmnd(0xC0);  /* set the voltage by sending C0 means VOP = 5V */
	N5110_Cmnd(0x07);  /* set the temp. coefficient to 3 */
	N5110_Cmnd(0x13);  /* set value of Voltage Bias System */
	N5110_Cmnd(0x20);  /* command set in basic mode */
	N5110_Cmnd(0x0C);  /* display result in normal mode */
}

void lcd_setXY(char x, char y)  /* set the column and row */
{
	N5110_Cmnd(x);
	N5110_Cmnd(y);
}

void N5110_clear()  /* clear the Display */
{
	SPI_SS_Enable();
	PORTB |= (1<<DC);
	for (int k=0; k<=503; k++)
	{
		SPI_Write(0x00);		
	}
	PORTB &= ~(1<<DC);
	SPI_SS_Disable();	
}

void N5110_image(const unsigned char *image_data)  /* clear the Display */
{
	SPI_SS_Enable();
	PORTB |= (1<<DC);
	for (int k=0; k<=503; k++)
	{
		SPI_Write(image_data[k]);
	}
	PORTB &= ~(1<<DC);
	SPI_SS_Disable();
}

int main()
{
	uint16_t adc_value = 0; // variable used to store the value read by the sensor
	uint16_t cleanAir = 0;	// clean air sensor value

	uint16_t alcValue = 0;
	uint16_t alcNormal = 40; // minimum alcohol concentration	
	uint16_t average = 0;
	uint16_t sum = 0;
	
	/*initializare ecran*/
	SPI_Init();
        N5110_init();
        N5110_clear();
        lcd_setXY(0x40,0x80);

	/* initiate adc */
	adc_init();
	char a[]="Wait\n";
	N5110_Data(a);

	/* calibrate sensor: standby */
	cleanAir = calibrate();
	N5110_clear();
	char b[]= "Suflati\n";
	N5110_Data(b);

	sum = 0;
	for (int i = 0; i < 50; i++) {
		adc_value = adc_read(0);
		sum += adc_value;
		_delay_ms(500);
	}
	average = (uint16_t)(sum/50);

	alcValue = convert(average, cleanAir, alcNormal);

	_delay_ms(1000);
	char c[15];
	N5110_clear();
	sprintf(c, "\nValue: %u\n", (unsigned int) alcValue);
	N5110_Data(c);

	_delay_ms(10000);
	
	return 0;
}
